/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.bc.filetransfer.listeners;

import java.io.File;
import java.util.logging.Level;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessagingException;
import javax.xml.namespace.QName;

import org.ow2.petals.bc.filetransfer.FileTransferComponent;
import org.ow2.petals.bc.filetransfer.FileTransferConstants;
import org.ow2.petals.bc.filetransfer.service.provide.AbstractFileService;
import org.ow2.petals.bc.filetransfer.service.provide.CheckFileService;
import org.ow2.petals.bc.filetransfer.service.provide.DelFileService;
import org.ow2.petals.bc.filetransfer.service.provide.DirService;
import org.ow2.petals.bc.filetransfer.service.provide.GetFileAsAttachmentService;
import org.ow2.petals.bc.filetransfer.service.provide.GetFileService;
import org.ow2.petals.bc.filetransfer.service.provide.MGetFileService;
import org.ow2.petals.bc.filetransfer.service.provide.MPutFileService;
import org.ow2.petals.bc.filetransfer.service.provide.PutFileService;
import org.ow2.petals.component.framework.api.Message;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.component.framework.logger.Utils;
import org.w3c.dom.Document;

import com.ebmwebsourcing.easycommons.lang.StringHelper;

/**
 * The file transfer BC JBI listener. 
 * <p>
 * Writes a file to hard disk on incoming JBI message.
 * </p>
 * 
 * @author Adrien Louis - EBM WebSourcing
 */
public class FileTransferJBIListener extends AbstractJBIListener {

    /**
     * Process the exchange : copy the received elements
     */
    @Override
    public boolean onJBIMessage(Exchange exchange) {
        if (exchange.isProviderRole()) {
            AbstractFileService service = null;
            // Fault ignored
            if (exchange.isActiveStatus()) {

                try {
                    validateExchange(exchange);
                    if (exchange.getFault() == null) {
                        final QName operation = exchange.getOperation();

                        final Document inMessageDoc = exchange.getInMessageContentAsDocument();
                        if (FileTransferConstants.GET_OPERATION.equalsIgnoreCase(operation
                                .getLocalPart())) {
                            service = new GetFileService(this.getExtensions(), this.getLogger());
                        } else if (FileTransferConstants.GET_ATTACHMENT_OPERATION
                                .equalsIgnoreCase(operation.getLocalPart())) {
                            service = new GetFileAsAttachmentService(this.getExtensions(),
                                    this.getLogger());
                        } else if (FileTransferConstants.MGET_OPERATION.equalsIgnoreCase(operation
                                .getLocalPart())) {
                            service = new MGetFileService(this.getExtensions(), this.getLogger());
                        } else if (FileTransferConstants.CHECK_FILE_OPERATION
                                .equalsIgnoreCase(operation.getLocalPart())) {
                            service = new CheckFileService(this.getExtensions(), this.getLogger());
                        } else if (FileTransferConstants.PUT_OPERATION.equalsIgnoreCase(operation
                                .getLocalPart())) {
                            service = new PutFileService(this.getExtensions(), this.getLogger());
                        } else if (FileTransferConstants.MPUT_OPERATION.equalsIgnoreCase(operation
                                .getLocalPart())) {
                            service = new MPutFileService(this.getExtensions(), this.getLogger());
                        } else if (FileTransferConstants.DEL_OPERATION.equalsIgnoreCase(operation
                                .getLocalPart())) {
                            service = new DelFileService(this.getExtensions(), this.getLogger());
                        } else if (FileTransferConstants.DIR_OPERATION.equalsIgnoreCase(operation
                                .getLocalPart())) {
                            service = new DirService(this.getExtensions(), this.getLogger());
                        } else {
                            final String message = "The operation '" + operation
                                    + "' is not recognized";
                            throw new MessagingException(message);
                        }
                        service.process(exchange, inMessageDoc);
                    } else {
                        exchange.setDoneStatus();
                    }
                } catch (MessagingException e) {
                    final String correlationID = (String) exchange
                            .getProperty(Message.CORRELATION_ID_KEY);
                    final StringBuilder builder = new StringBuilder();
                    builder.append("[Exchange Id:" + exchange.getExchangeId());
                    if (!StringHelper.isNullOrEmpty(correlationID)) {
                        builder.append(";Correlation Id:" + correlationID);
                    }
                    builder.append("] - ");
                    builder.append(e.getClass().getSimpleName() + " : " + e.getMessage());
                    this.getLogger().log(Level.WARNING, builder.toString());
                    exchange.setError(e);
                }
            }
        } else if (!ExchangeStatus.ACTIVE.equals(exchange.getStatus())) {
            final File transferedfile = ((FileTransferComponent) this.component)
                    .getTransferedFile(exchange);
            if (transferedfile != null) {
                this.getLogger().info(
                        "Received acknowledgment '" + exchange.getStatus().toString()
                                + "' for transfered file '" + transferedfile.getAbsolutePath()
                                + "'. Delete it");
                Utils.addMonitEndOrFailureTrace(this.getLogger(), exchange,
                        exchange.getFlowAttributes());
                if (!transferedfile.delete()) {
                    this.getLogger().log(Level.WARNING,
                            "Failed to delete file '" + transferedfile.getAbsolutePath() + "'");
                }
            }
        }
        return true;
    }

    /**
     * 
     * Validate the operation.
     * 
     * @param exchange
     * @throws MessagingException
     */
    private static final void validateExchange(final Exchange exchange) throws MessagingException {
        final QName operation = exchange.getOperation();
        // Null operation
        if (operation == null) {
            throw new MessagingException(
                    "The JBI operation must be set by the consumer of the service.");
        }
        // if no namespace is provided we use the default namespace.
        if (StringHelper.isNullOrEmpty(operation.getNamespaceURI())) {
            exchange.setOperation(new QName(FileTransferConstants.FILETRANSFER_SERVICE_NS,
                    operation.getLocalPart()));
        }
    }
}
