/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.bc.filetransfer.service.consume;

import java.io.File;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.Future;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.RejectedExecutionException;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.petals.bc.filetransfer.FileTransferComponent;
import org.ow2.petals.bc.filetransfer.listeners.FileTransferExternalListener;
import org.ow2.petals.component.framework.su.AbstractServiceUnitManager;
import org.ow2.petals.component.framework.su.ServiceUnitDataHandler;

/**
 * Watch a folder and create a new thread for each new file.
 * @author Mathieu CARROLLE - EBM WebSourcing
 */
public class ExternalEventPoller extends TimerTask {

    private final FileTransferComponent component;

    /**
     * Consume configuration
     */
    private final ConsumeConfiguration configuration;

    /**
     * FileTransferExternalListener instance use to access logger and CDK method
     * like send()
     */
    private final FileTransferExternalListener consumeContext;

    private final Logger logger;

    /**
     * The maximum pool size have no effect/corePoolSize determine the maximun
     * number of thread.<br/>
     * Idle core thread are remove after the defined timeout. <br/>
     * Use LinkedBlockingQueue() to store pending task when all core thread are
     * busy.
     */
    private final ThreadPoolExecutor poolExecutor;

    /**
     * The timer used for task scheduling
     */
    private Timer timer;

    public ExternalEventPoller(ConsumeConfiguration configuration,
            FileTransferExternalListener consumeContext) {
        this.configuration = configuration;
        this.consumeContext = consumeContext;
        this.component = (FileTransferComponent) consumeContext.getComponent();
        this.logger = consumeContext.getLogger();
        final ServiceUnitDataHandler dtHandler = ((AbstractServiceUnitManager) this.component
                .getServiceUnitManager()).getSUDataHandlerForConsumes(this.consumeContext
                .getConsumes());
        this.timer = new Timer("petals-bc-filetransfer - External Event Poller Thread ["
                + dtHandler.getName() + "]");
        this.poolExecutor = new ThreadPoolExecutor(configuration.getCorePoolSize(),
                configuration.getCorePoolSize(), configuration.getCorePoolTimeout(),
                TimeUnit.MILLISECONDS, new LinkedBlockingQueue<Runnable>(),
                new ExternalEventProcessorFactory());
        this.poolExecutor.allowCoreThreadTimeOut(true);
    }

    @Override
    public void run() {
        this.component.cleanPendingProcessors();
        if (configuration.getPolledDirectory().canRead()) {
            File[] list = configuration.getPolledDirectory().listFiles(
                    configuration.getFilePattern());
            if (list != null && list.length > 0) {
                for (File file : list) {
                    try {
                        Map<String, Future<?>> pendingProcessor = this.component
                                .getPendingProcessors();
                        if (!pendingProcessor.containsKey(file.getName())) {
                            Future<?> fileProcessor = poolExecutor
                                    .submit(new ExternalEventProcessor(file, this));
                            pendingProcessor.put(file.getName(), fileProcessor);
                        }
                    } catch (RejectedExecutionException e) {
                        this.logger.info("Max ExternalEventProcessor pool size reached");
                    } catch (Throwable e) {
                        // catch throwable to avoid the death of this thread
                        this.logger.info(e.getClass().getName() + " : " + e.getMessage());
                    }
                }
            }
        } else {
            this.logger.info("Failed to read polled directory : ["
                    + configuration.getPolledDirectory().getAbsolutePath() + "]");
        }
    }

    /**
     * Start this poller
     */
    public void startPolling() {
        this.logger.info("start polling directory : ["
                + configuration.getPolledDirectory().getAbsolutePath() + "]");
        if (this.logger.isLoggable(Level.FINEST)) {
            this.logger.finest("Configuration: [Transfer Mode: " + configuration.getTransferMode()
                    + "; Polling Period: " + configuration.getPeriod()
                    + "ms; Use message template: " + configuration.isBaseMsgOn() + "]");
        }
        this.timer.schedule(this, this.configuration.getDelay(), this.configuration.getPeriod());
    }

    /**
     * Cancel and purge this timer.<br/>
     * Shutdown all ExternalEventProcessor thread (wait the end of all current
     * tasks and don't accept new Task)
     */
    public void stopPolling() {
        this.logger.info("stop polling directory : "
                + this.configuration.getPolledDirectory().getAbsolutePath());
        this.poolExecutor.shutdown();
        this.timer.cancel();
        this.timer.purge();
        this.timer = null;
    }

    protected ConsumeConfiguration getConfiguration() {
        return configuration;
    }

    protected FileTransferExternalListener getConsumeContext() {
        return consumeContext;
    }

    protected Logger getLogger() {
        return logger;
    }
}
