/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.bc.filetransfer.service.consume;

import java.io.File;
import java.io.IOException;
import java.util.logging.Logger;

import javax.jbi.messaging.MessageExchange.Role;
import javax.jbi.messaging.MessagingException;

import org.apache.commons.io.FileUtils;
import org.ow2.petals.bc.filetransfer.FileTransferComponent;
import org.ow2.petals.bc.filetransfer.FileTransferConstants;
import org.ow2.petals.bc.filetransfer.FileTransferConsumeFlowStepBeginLogData;
import org.ow2.petals.bc.filetransfer.listeners.FileTransferExternalListener;
import org.ow2.petals.bc.filetransfer.util.FileTransferUtils;
import org.ow2.petals.commons.log.Level;
import org.ow2.petals.commons.log.PetalsExecutionContext;
import org.ow2.petals.component.framework.api.Message.MEPConstants;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.logger.Utils;
import org.w3c.dom.Document;

import com.ebmwebsourcing.easycommons.lang.StringHelper;
import com.ebmwebsourcing.easycommons.thread.ExecutionContext;

/**
 * Process a file.
 * @author Mathieu CARROLLE - EBM WebSourcing
 */
public class ExternalEventProcessor implements Runnable {

    private final ConsumeConfiguration configuration;

    private final FileTransferExternalListener consumeContext;

    private final Logger logger;

    private final File originalFile;

    public ExternalEventProcessor(final File file, final ExternalEventPoller pollerContext) {
        this.originalFile = file;
        this.consumeContext = pollerContext.getConsumeContext();
        this.configuration = pollerContext.getConfiguration();
        this.logger = pollerContext.getLogger();
    }

    @Override
    public void run() {
        Document finalMsg = null;
        Exchange exchange = null;
        final String fileName = originalFile.getName();

        File backupFile = null;
        final File lockFile = new File(this.originalFile.getAbsolutePath() + ".lock");
        PetalsExecutionContext.initFlowInstanceId();
        PetalsExecutionContext.nextFlowStepId();
        try {
            // try to create lockFile
            if (lockFile.createNewFile()) {
                try {
                    // Verify if the file to process is always available
                    if (this.originalFile.exists()) {
                        this.logger.info("process file : " + this.originalFile.getName());
                        // move file in a work or backup dir before processing
                        if (configuration.getBackupDirectory() == null) {
                            backupFile = this.moveFile(originalFile,
                                    new File(System.getProperty("java.io.tmpdir")));
                        } else {
                            backupFile = this.moveFile(originalFile,
                                    configuration.getBackupDirectory());
                        }
                        // Build message
                        if (configuration.isBaseMsgOn()) {
                            if (this.logger.isLoggable(Level.FINE)) {
                                this.logger
                                        .fine("reading and setting the processed file in the base message.");
                            }
                            finalMsg = FileTransferUtils.createMessageOnBaseMsg(backupFile,
                                    fileName, configuration.getBaseMsg(),
                                    configuration.getTransferMode());
                        } else {
                            if (this.logger.isLoggable(Level.FINE)) {
                                this.logger
                                        .fine("reading and setting the processed file in the default message.");
                            }
                            finalMsg = FileTransferUtils.createMessageOnDefaultMsg(backupFile,
                                    fileName, configuration.getTransferMode());
                        }
                        // Send message
                        exchange = consumeContext.createConsumeExchange(
                                consumeContext.getConsumes(), MEPConstants.IN_ONLY_PATTERN);
                        exchange.setInMessageContent(finalMsg);

                        String flowInstanceId = ExecutionContext.getProperties().getProperty(
                                PetalsExecutionContext.FLOW_INSTANCE_ID_PROPERTY_NAME);
                        String flowStepId = ExecutionContext.getProperties().getProperty(
                                PetalsExecutionContext.FLOW_STEP_ID_PROPERTY_NAME);
                        Consumes consumes = consumeContext.getConsumes();
                        logger.log(
                                Level.MONIT,
                                "",
                                new FileTransferConsumeFlowStepBeginLogData(flowInstanceId,
                                        flowStepId, StringHelper.nonNullValue(consumes
                                                .getInterfaceName()), StringHelper
                                                .nonNullValue(consumes.getServiceName()), consumes
                                                .getEndpointName(), StringHelper
                                                .nonNullValue(consumes.getOperation()),
                                        this.originalFile.getAbsolutePath()));

                        if (configuration.getTransferMode().equals(
                                FileTransferConstants.ENUM_TRANSFER_MODE_ATTACHMENT)) {
                            this.logger.finest("add attachment");
                            FileTransferUtils.addFileAsAttachment(backupFile, fileName,
                                    exchange.getInMessage());
                        }

                        consumeContext.send(exchange);
                        if (configuration.getBackupDirectory() == null) {
                            ((FileTransferComponent) consumeContext.getComponent())
                                    .addTransferedFile(exchange, backupFile);
                        }
                    } else {
                        this.logger.fine("[" + this.originalFile.getName()
                                + "] : file no longer exists");
                    }

                } catch (MessagingException me) {
                    this.logger.log(Level.INFO, "Can't process file " + fileName, me);
                    Utils.addMonitFailureTrace(this.logger,
                            PetalsExecutionContext.getFlowAttributes(), me.getMessage(),
                            Role.CONSUMER);
                } catch (Exception e) {
                    this.logger.log(Level.INFO, "Can't process file " + fileName, e);
                }

                finally {
                    // Delete lock file
                    if (!lockFile.delete()) {
                        this.logger.fine("Unable to delete lock file: ["
                                + lockFile.getAbsolutePath() + "]");
                    }
                }
            } else {
                this.logger.finest("The lock file already exists. [" + fileName
                        + "] is processed by another petals-bc-filetransfer instance");
            }
        } catch (IOException e) {
            this.logger.info("Unable to create lock file: " + e.getMessage());
        }
    }

    /**
     * Move the file to the waste directory
     * 
     * @param file
     *            original file to process
     * @param destDir
     *            working file directory
     * @return the new file reference
     * @throws IOException
     *             fail to move the file
     */
    private File moveFile(final File file, final File destDir) throws IOException {
        // Check file's size
        long fileSize;
        do {
            fileSize = file.length();
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        } while (file.length() > fileSize);

        this.logger.fine("Move file from " + file.getAbsolutePath() + " to "
                + destDir.getAbsolutePath());
        final String target = "backup-" + FileTransferUtils.addDateToFileName(file.getName());
        final File newFile = new File(destDir, target);
        final boolean success = file.renameTo(newFile);
        if (!success) {
            // if the renameTO method is processed between 2 different File
            // System, the Java method may return false. In this case we try to
            // copy the file and remove the old one.
            FileUtils.copyFile(file, newFile);
            file.delete();
        }
        return newFile;
    }
}
