/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.bc.filetransfer.service.provide;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.messaging.MessagingException;
import javax.xml.namespace.QName;

import org.ow2.petals.bc.filetransfer.FileTransferConstants;
import org.ow2.petals.bc.filetransfer.util.FileTransferUtils;
import org.ow2.petals.component.framework.api.Message.MEPConstants;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.util.SourceUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

import com.ebmwebsourcing.easycommons.xml.XMLHelper;

/***
 * Check if the specified file exist(absolute and relative path are allowed).
 * <p>
 * Request exemple : <q0:checkFile
 * xmlns:q0="http://petals.ow2.org/components/filetransfer/version-2">
 * <q0:filepath>/mypath/myfile.xml</q0:filepath> </q0:checkFile>
 * </p>
 * 
 * @author Mathieu CARROLLE - EBM WebSourcing
 */
public class CheckFileService extends AbstractFileService {

	public CheckFileService(ConfigurationExtensions extensions, Logger logger) {
		super(extensions, logger);
	}

	@Override
	public void process(Exchange exchange, Document inMessageDoc)
			throws MessagingException {
		boolean exist = false;
		this.logger.info("JBI exchange receive - process checkFile operation");
		if (!exchange.isInOutPattern()) {
			throw new MessagingException(
					FileTransferUtils.getValidMepMessageForOperation(
							FileTransferConstants.CHECK_FILE_OPERATION,
							MEPConstants.IN_OUT_PATTERN));
		}
		try {
			final String filepath = this.extractFilePath(inMessageDoc);
			File file = new File(filepath);
			if (file != null && !file.isAbsolute()) {
				if (this.logger.isLoggable(Level.FINEST)) {
					this.logger
							.log(Level.FINEST,
									"File ["
											+ filepath
											+ "] is not an absolute path, resolving the read-directory as a parent directory");
				}
				final File directory = FileTransferUtils
						.getFolderPath(extensions);
				file = new File(directory, filepath);
			}
			exist = file.exists();

			this.addMonitBeginStepExtTrace(filepath, FileTransferUtils
					.getFolderPath(extensions).getName());

			final String response = this.generateResponse(exist,
					exchange.getOperation(), file);
			exchange.getOutMessage().setContent(
					SourceUtil.createSource(response));
			this.addMonitEndStepExtTrace();
			if (this.logger.isLoggable(Level.FINEST)) {
				this.logger.log(
						Level.FINEST,
						"resolved file path (Canonical form) : "
								+ file.getCanonicalPath());
			}
		} catch (IOException e) {
			this.logger.log(Level.WARNING, e.getMessage());
			FileTransferUtils.setIOFaultOnExchange(exchange, e.getMessage());
		} catch (PEtALSCDKException e) {
			this.logger.log(Level.WARNING, e.getMessage());
			FileTransferUtils.setIOFaultOnExchange(exchange, e.getMessage());
		}
	}

	/**
	 * Exctract the xml filename element in the incomming jbi message
	 * 
	 * @param exchange
	 * @return content of the filepath element
	 * @throws MessagingException
	 * @throws FileNotFoundException
	 */
	private String extractFilePath(Document inMessageDoc)
			throws MessagingException, FileNotFoundException {
		final Node rootNode = inMessageDoc.getFirstChild();
		Node pattern = null;
		if (rootNode != null) {
			pattern = XMLHelper.findChild(rootNode, null,
					FileTransferConstants.PARAM_FILENAME, false);
		}
		if (pattern == null) {
			throw new FileNotFoundException("Can't find "
					+ FileTransferConstants.PARAM_FILENAME + " node");
		}
		return pattern.getTextContent();
	}

	/**
	 * Generate the jbi response message
	 * 
	 * @param checkFile
	 * @param exchange
	 * @param filepath
	 * @return
	 */
	private String generateResponse(final Boolean checkFile,
			final QName operation, final File file) {
		final StringBuilder builder = new StringBuilder();
		builder.append("<tns:" + operation.getLocalPart()
				+ "Response xmlns:tns='");
		builder.append(operation.getNamespaceURI());
		builder.append("'>");
		builder.append("<tns:filename>");
		builder.append(file.getAbsolutePath());
		builder.append("</tns:filename>");
		builder.append("<tns:exist>");
		builder.append(checkFile ? "true" : "false");
		builder.append("</tns:exist>");
		builder.append("</tns:" + operation.getLocalPart() + "Response>");
		return builder.toString();
	}

}
