/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.bc.filetransfer.service.provide;

import java.io.File;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.activation.FileDataSource;
import javax.jbi.messaging.MessagingException;

import org.ow2.petals.bc.filetransfer.FileTransferConstants;
import org.ow2.petals.bc.filetransfer.util.FileTransferUtils;
import org.ow2.petals.component.framework.api.Message;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.util.FileNamePatternUtil;
import org.ow2.petals.component.framework.util.FileNamePatternUtil.RegexFileNameFilter;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

import com.ebmwebsourcing.easycommons.lang.StringHelper;
import com.ebmwebsourcing.easycommons.xml.XMLHelper;

/**
 * Get the file in the configured directory(parameter folder) and set it as
 * attachment on the out-message.
 * 
 * @author Mathieu CARROLLE - EBM WebSourcing
 */
public class GetFileAsAttachmentService extends AbstractFileService {

	public GetFileAsAttachmentService(ConfigurationExtensions extensions,
			Logger logger) {
		super(extensions, logger);
	}

	private String fileName = null;

	@Override
	public void process(Exchange exchange, Document inMessageDoc)
			throws MessagingException {
		this.logger
				.info("JBI exchange received - Process GET AS ATTACHMENT operation");
		// Checks the MEP of the incoming message
		if (!exchange.isInOutPattern()) {
			throw new MessagingException(
					FileTransferUtils.getValidMepMessageForOperation(
							FileTransferConstants.GET_ATTACHMENT_OPERATION,
							Message.MEPConstants.IN_OUT_PATTERN));
		}

		// retrieve the filename to get
		final Node filePatternNode = XMLHelper.findChild(
				inMessageDoc.getFirstChild(), null,
				FileTransferConstants.PARAM_FILENAME, true);
		String filePattern = null;
		if (filePatternNode != null) {
			filePattern = filePatternNode.getTextContent();
		}
		if (StringHelper.isNullOrEmpty(filePattern))
			FileTransferUtils.setMissingElementFaultOnExchange(exchange,
					FileTransferConstants.PARAM_FILENAME);
		else
			try {
				final File readDirectory = FileTransferUtils
						.getFolderPath(this.extensions);
				this.logger.log(Level.INFO, "read files from : "
						+ readDirectory.getAbsolutePath());
				this.logger.log(Level.INFO, "file pattern : " + filePattern);
				this.logger.log(Level.INFO, "read files from : "
						+ readDirectory.getAbsolutePath());

				this.addMonitBeginStepExtTrace(filePattern,
						readDirectory.getName());

				final DataHandler file = this.getAsAttachment(filePattern,
						readDirectory);
				exchange.setOutMessageAttachment(this.fileName, file);
				final Document outDoc = FileTransferUtils.generateMTOMResponse(
						this.fileName, exchange.getOperation());
				exchange.setOutMessageContent(outDoc);
				this.addMonitEndStepExtTrace();
			} catch (final Exception e) {
				FileTransferUtils.setIOFaultOnExchange(
						exchange,
						"Can not get file [" + filePattern + "]. Cause : "
								+ e.getMessage());
			}
		this.logger.info("GET AS ATTACHMENT operation ended");
	}

	/***
	 * 
	 * @param filePattern
	 * @param readDirecory
	 * @return
	 * @throws IOException
	 */
	private DataHandler getAsAttachment(String filePattern, File readDirectory)
			throws IOException {
		final DataHandler file;
		final RegexFileNameFilter filter = FileNamePatternUtil.getInstance()
				.buildFileNameFilterFromWildChar(filePattern);
		final File[] files = readDirectory.listFiles(filter);
		if (files.length > 0) {
			if (!FileTransferUtils.checkAccess(files[0])) {
				throw new IOException("File " + files[0].getName()
						+ " is locked.Permission denied");
			}
			this.fileName = files[0].getName();
			// move file in a work or backup dir before processing
			final File processedFile = FileTransferUtils.moveFile(files[0],
					this.extensions);
			final DataSource source = new FileDataSource(processedFile);
			file = new DataHandler(source);
		} else {
			throw new IOException("file : '" + filePattern + "' not found");
		}
		return file;
	}
}
