/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.bc.filetransfer.service.provide;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.messaging.MessagingException;
import javax.xml.namespace.QName;

import org.ow2.petals.bc.filetransfer.FileTransferConstants;
import org.ow2.petals.bc.filetransfer.util.FileTransferUtils;
import org.ow2.petals.component.framework.api.Message;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.util.XMLUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import com.ebmwebsourcing.easycommons.lang.StringHelper;
import com.ebmwebsourcing.easycommons.xml.XMLHelper;

/**
 * Get the file in the configured directory (parameter : folder) and set it in
 * the out-message payload.
 * 
 * @author Mathieu CARROLLE - EBM WebSourcing
 */
public class GetFileService extends AbstractFileService {

	public GetFileService(final ConfigurationExtensions extensions,
			final Logger logger) {
		super(extensions, logger);
	}

	/**
	 * Process the exchange : get the file in the configured directory, <br>
	 * using filename set in the exchange <br>
	 * file is returned as OUT response (assume the file is an XML file) <br>
	 * return the first file <br>
	 * the file is move/deleted after read
	 * 
	 * @throws JBIException
	 */
	@Override
	public void process(final Exchange exchange, final Document inMessageDoc)
			throws MessagingException {
		if (this.logger.isLoggable(Level.INFO)) {
			this.logger.info("JBI exchange received - Process GET operation");
		}
		// Checks the MEP of the incoming message
		if (!exchange.isInOutPattern()) {
			throw new MessagingException(
					FileTransferUtils.getValidMepMessageForOperation(
							FileTransferConstants.GET_OPERATION,
							Message.MEPConstants.IN_OUT_PATTERN));
		}
		// retrieve the filename to get
		final Node filePatternNode = XMLHelper.findChild(
				inMessageDoc.getFirstChild(), null,
				FileTransferConstants.PARAM_FILENAME, false);
		String filePattern = null;
		if (filePatternNode != null) {
			filePattern = filePatternNode.getTextContent();
		}
		if (StringHelper.isNullOrEmpty(filePattern)) {
			FileTransferUtils.setMissingElementFaultOnExchange(exchange,
					FileTransferConstants.PARAM_FILENAME);
		} else {
			try {
				final File readDirectory = FileTransferUtils
						.getFolderPath(this.extensions);
				this.logger.log(Level.INFO, "read files from : "
						+ readDirectory.getAbsolutePath());
				this.logger.log(Level.INFO, "file pattern : " + filePattern);

				this.addMonitBeginStepExtTrace(filePattern,
						readDirectory.getName());
				final Document outContent = this.get(filePattern,
						readDirectory, exchange.getOperation());
				exchange.setOutMessageContent(outContent);

				this.addMonitEndStepExtTrace();
			} catch (final Exception e) {
				FileTransferUtils.setIOFaultOnExchange(
						exchange,
						"Can not get file [" + filePattern + "]. Cause : "
								+ e.getMessage());
			}
		}
		this.logger.log(Level.INFO, "GET service is correctly terminated");
	}

	/**
	 * Get the first file corresponding to filepattern and read his content.
	 * 
	 * @param filePattern
	 * @param readDirectory
	 * @return the xml document
	 * @throws IOException
	 */
	private Document get(final String filePattern, File readDirectory,
			final QName operation) throws IOException {
		final Document result;
		final File[] files = FileTransferUtils.listFiles(readDirectory,
				filePattern);
		if (files.length > 0) {
			this.logger.log(Level.INFO, "file found : " + files[0].getName());
			if (!FileTransferUtils.checkAccess(files[0])) {
				throw new IOException("File " + files[0].getName()
						+ " is locked.Permission denied");
			}

			// move file in a work or backup dir before processing
			final File processedFile = FileTransferUtils.moveFile(files[0],
					this.extensions);
			//
			result = getAsDocument(processedFile, operation);
		} else {
			throw new IOException("file : '" + filePattern + "' not found");
		}
		return result;
	}

	/**
	 * If file content, isn't an xml content type so an exception is thrown.
	 * 
	 * @param processedFile
	 * @return
	 * @throws IOException
	 */
	private Document getAsDocument(File processedFile, QName operation)
			throws IOException {
		final Document doc;
		if (this.logger.isLoggable(Level.FINEST)) {
			logger.finest("start receiving file " + processedFile.getName());
		}
		final InputStream stream = new BufferedInputStream(new FileInputStream(
				processedFile));
		try {
			doc = XMLUtil.loadDocument(stream);
			final Element rootElt = doc.createElementNS(
					operation.getNamespaceURI(),
					"ver:" + operation.getLocalPart() + "Response");
			rootElt.appendChild(doc.getDocumentElement());
			doc.appendChild(rootElt);
		} catch (SAXException e) {
			throw new IOException("processed file '" + processedFile.getName()
					+ "' is not a valid xml file : " + e.getMessage());
		} finally {
			stream.close();
		}

		if (this.logger.isLoggable(Level.FINEST)) {
			logger.finest(processedFile.getName() + " set as content");
		}
		return doc;
	}
}
