/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.bc.filetransfer.service.provide;

import java.io.File;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.transform.dom.DOMSource;

import org.ow2.petals.bc.filetransfer.FileTransferConstants;
import org.ow2.petals.bc.filetransfer.util.FileTransferUtils;
import org.ow2.petals.component.framework.api.Message.MEPConstants;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.util.FileNamePatternUtil;
import org.ow2.petals.component.framework.util.FileNamePatternUtil.RegexFileNameFilter;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

/***
 * Get files as JBI attachment.
 * <p>
 * The working directory is determined by the parameter 'folder'.
 * </p>
 * 
 * @author Mathieu CARROLLE - EBM WebSourcing
 */
public class MGetFileService extends AbstractFileService {

	public MGetFileService(ConfigurationExtensions extensions, Logger logger) {
		super(extensions, logger);
	}

	@Override
	public void process(Exchange exchange, Document inMessageDoc)
			throws MessagingException {
		if (this.logger.isLoggable(Level.INFO)) {
			this.logger.info("JBI exchange received - Process MGET operation");
		}
		if (!exchange.isInOutPattern()) {
			throw new MessagingException(
					FileTransferUtils.getValidMepMessageForOperation(
							FileTransferConstants.MGET_OPERATION,
							MEPConstants.IN_OUT_PATTERN));
		}
		// retrieve filenames to get
		final NodeList list = inMessageDoc.getFirstChild().getChildNodes();
		final List<String> filePattern = new LinkedList<String>();
		for (int index = 0; index < list.getLength(); index++) {
			if (FileTransferConstants.PARAM_FILENAME.equalsIgnoreCase(list
					.item(index).getLocalName())) {
				filePattern.add(list.item(index).getTextContent());
			}
		}
		if (filePattern.size() == 0) {
			FileTransferUtils.setMissingElementFaultOnExchange(exchange,
					FileTransferConstants.PARAM_FILENAME);
		} else {
			File readDirectory = null;
			try {
				readDirectory = FileTransferUtils
						.getFolderPath(this.extensions);
				this.logger.log(Level.INFO, "read files from : "
						+ readDirectory.getAbsolutePath());

				final StringBuilder builder = new StringBuilder();
				for (int i = 0; i < filePattern.size(); i++) {
					builder.append("(" + filePattern.get(i) + ")");
					if ((i + 1) < filePattern.size()) {
						builder.append("|");
					}
				}

				this.addMonitBeginStepExtTrace(builder.toString(),
						readDirectory.getName());

				final NormalizedMessage outMessage = exchange.getOutMessage();
				final List<String> fileName = this.mGet(filePattern,
						readDirectory, outMessage);
				final Document outDoc = FileTransferUtils
						.generateMTOMListResponse(fileName,
								exchange.getOperation());
				outMessage
						.setContent(new DOMSource(outDoc.getDocumentElement()));

				this.addMonitEndStepExtTrace();
			} catch (IOException e) {
				FileTransferUtils.setIOFaultOnExchange(
						exchange,
						"Can not get file [" + filePattern + "] Cause : "
								+ e.getMessage());
			}
		}
		if (this.logger.isLoggable(Level.FINEST)) {
			this.logger.finest("MGET operation ended");
		}
	}

	/***
	 * 
	 * @param fileWildCardPatterns
	 * @param readDirectory
	 * @return
	 * @throws IOException
	 * @throws MessagingException
	 * @throws IOException
	 */
	private List<String> mGet(List<String> fileWildCardPatterns,
			File readDirectory, NormalizedMessage outMessage)
			throws MessagingException, IOException {
		final List<String> retrievedFileName = new LinkedList<String>();
		final List<RegexFileNameFilter> fileFilters = new LinkedList<RegexFileNameFilter>();
		for (String pattern : fileWildCardPatterns) {
			fileFilters.add(FileNamePatternUtil.getInstance()
					.buildFileNameFilterFromWildChar(pattern));
		}
		final RegexFileNameFilter filter = FileNamePatternUtil.getInstance()
				.buildFileNameFilterFromFilters(fileFilters);
		final File[] ls = readDirectory.listFiles();
		String filename;
		for (File file : ls) {
			if (file.isFile() && filter.accept(null, file.getName())) {
				if (FileTransferUtils.checkAccess(file)) {
					filename = file.getName();
					// move file in a work or backup dir before processing
					final File processedFile = FileTransferUtils.moveFile(file,
							this.extensions);
					final DataHandler attachment = new DataHandler(
							processedFile.toURI().toURL());
					outMessage.addAttachment(filename, attachment);
					retrievedFileName.add(filename);
				} else {
					this.logger.warning("File " + file.getName()
							+ " is locked.Permission denied");
				}
			}
		}
		return retrievedFileName;
	}
}
