/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.bc.filetransfer.service.provide;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Iterator;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.jbi.messaging.MessagingException;

import org.ow2.petals.bc.filetransfer.FileTransferConstants;
import org.ow2.petals.bc.filetransfer.util.FileTransferUtils;
import org.ow2.petals.bc.filetransfer.util.TransferReport;
import org.ow2.petals.component.framework.api.Message.MEPConstants;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.util.MtomUtil;
import org.w3c.dom.Document;

/***
 * Copy attachments of the JBI in-message into the specified directory.(parameter : write-directory).
 * <p>
 * If one copy process fails, a report is generated, record in petals-esb and and
 * error status is set on the exchange. The name of the transferred file is the
 * name of the attachment.
 * </p>
 * 
 * @author Mathieu CARROLLE - EBM WebSourcing
 */
public class MPutFileService extends AbstractFileService {

	public MPutFileService(ConfigurationExtensions extensions, Logger logger) {
		super(extensions, logger);
	}

	@Override
	public void process(Exchange exchange, Document inMessageDoc)
			throws MessagingException {
		final File directory;
		if (this.logger.isLoggable(Level.INFO)) {
			this.logger.info("JBI exchange received - Process MPUT operation");
		}

		if (!exchange.isInOnlyPattern()) {
			throw new MessagingException(
					FileTransferUtils.getValidMepMessageForOperation(
							FileTransferConstants.MPUT_OPERATION,
							MEPConstants.IN_ONLY_PATTERN));
		}

		// Check if the message has attachements
		if (exchange.getInMessageAttachments().isEmpty()) {
			final StringBuilder errorSB = new StringBuilder();
			errorSB.append("One or more attachments expected.");
			throw new MessagingException(errorSB.toString());
		}
		final TransferReport transferReport = new TransferReport();
		final Map<String, DataHandler> attachments = MtomUtil.getAttachments(
				inMessageDoc, exchange.getInMessage());
		try {
			directory = FileTransferUtils.getFolderPath(extensions);
		} catch (IOException e) {
			throw new MessagingException(e.getMessage());
		}

		final StringBuilder builder = new StringBuilder();
		final Iterator<String> it = attachments.keySet().iterator();
		while (it.hasNext()) {
			builder.append("(" + it.next() + ")");
			if (it.hasNext()) {
				builder.append("|");
			}
		}

		this.addMonitBeginStepExtTrace(builder.toString(), directory.getName());

		for (String name : attachments.keySet()) {
			final DataHandler dataHandler = exchange
					.getInMessageAttachment(name);
			copyAttachment(dataHandler, name, directory, transferReport);
		}

		if (transferReport != null && transferReport.containsFailedTransfer()) {
			final String report = transferReport.generateRawReport();
			this.logger.log(Level.WARNING, report);
			throw new MessagingException(report);
		}
		if (this.logger.isLoggable(Level.FINEST)) {
			this.logger.finest("MPUT operation ended");
		}
	}

	private void copyAttachment(final DataHandler attachment,
			final String name, final File directory,
			final TransferReport transferReport) {
		// @TODO : Add prefix param to prefix or not filename
		// final String fileName = FileTransferUtils.addDateToFileName(name);
		final File target = new File(directory, name);
		FileOutputStream os = null;
		try {
			os = new FileOutputStream(target);
			this.logger.info("Attachment transfer to file : "
					+ target.getAbsolutePath());
			attachment.writeTo(os);
			transferReport.addTransferedFile(target, null);
		} catch (final IOException e) {
			this.logger.warning(e.getMessage());
			transferReport.addTransferedFile(target, e);
		} finally {
			try {
				if (os != null) {
					os.close();
				}
			} catch (IOException e) {
				// nothing
			}
		}
	}
}
