/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.bc.filetransfer.service.provide;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.messaging.MessagingException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.ow2.petals.bc.filetransfer.FileTransferConstants;
import org.ow2.petals.bc.filetransfer.util.FileTransferUtils;
import org.ow2.petals.component.framework.api.Message.MEPConstants;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

import com.ebmwebsourcing.easycommons.pooling.PoolException;
import com.ebmwebsourcing.easycommons.xml.Transformers;
import com.ebmwebsourcing.easycommons.xml.XMLHelper;

/***
 * Writing JBI in-message payload in a file. Xml and non-xml content can be process.
 * @author Mathieu CARROLLE - EBM WebSourcing
 */
public class PutFileService extends AbstractFileService {

	public PutFileService(ConfigurationExtensions extensions, Logger logger) {
		super(extensions, logger);
	}

	@Override
	public void process(Exchange exchange, Document inMessageDoc)
			throws MessagingException {
		if (this.logger.isLoggable(Level.INFO)) {
			this.logger.info("JBI exchange received - Process PUT operation");
		}
		// Checks the MEP of the incoming message
		if (!exchange.isInOnlyPattern()) {
			throw new MessagingException(
					FileTransferUtils.getValidMepMessageForOperation(
							FileTransferConstants.PUT_OPERATION,
							MEPConstants.IN_ONLY_PATTERN));
		}
		// Check the root node
		final Node root = inMessageDoc.getFirstChild();
		if (root == null
				|| !FileTransferConstants.PUT_OPERATION.equalsIgnoreCase(root
						.getLocalName())) {
			throw new MessagingException("Missing element : "
					+ FileTransferConstants.PUT_OPERATION);
		}
		try {

			final Node filePatternNode = XMLHelper.findChild(
					inMessageDoc.getFirstChild(), null,
					FileTransferConstants.PARAM_FILENAME, false);
			String filePattern = null;
			if (filePatternNode != null) {
				filePattern = filePatternNode.getTextContent();
			}

			this.addMonitBeginStepExtTrace(filePattern, FileTransferUtils
					.getFolderPath(extensions).getName());

			this.copySource(exchange, root.getFirstChild());
		} catch (TransformerException e) {
			final StringBuilder errorSB = new StringBuilder();
			errorSB.append("Can not put non-xml content : TransformerException : ");
			errorSB.append(e.getMessage());
			throw new MessagingException(errorSB.toString());
		} catch (IOException e) {
			final StringBuilder errorSB = new StringBuilder();
			errorSB.append("Can not put XML content : IOException : ");
			errorSB.append(e.getMessage());
			throw new MessagingException(errorSB.toString());
		} catch (PoolException e) {
			final StringBuilder errorSB = new StringBuilder();
			errorSB.append("Unable to get a XML transformer : PoolException : ");
			errorSB.append(e.getMessage());
			throw new MessagingException(errorSB.toString());
		}
		if (this.logger.isLoggable(Level.FINEST)) {
			this.logger.finest("PUT operation ended");
		}
	}

	/***
	 * Copy the xml source as a File.
	 * 
	 * @param exchange
	 *            in-message
	 * @param contentNode
	 *            Content node represent the first child node of the root node :
	 *            'put'
	 * @throws IOException
	 *             impossible to write to the file
	 * @throws TransformerException
	 *             if an unrecoverable error occurs during the course of the
	 *             transformation
	 * @throws PoolException
	 *             This method uses a transformer got from a transformer pool.
	 *             This unchecked exception is thrown if the current thread
	 *             waits for a transformer of the pool and is interrupted when
	 *             the pool policy is WAIT or if there is no more available
	 *             transformer in the transformer pool when the pool policy is
	 *             REJECT
	 */
	private void copySource(final Exchange exchange, Node contentNode)
			throws IOException, TransformerException {
		final File directory = FileTransferUtils.getFolderPath(extensions);
		final String fileName = getTargetFileName(exchange);

		final File target = new File(directory, fileName);
		this.logger.info("Source transfer to file : "
				+ target.getAbsolutePath());
		final Source source = new DOMSource(contentNode);
		final FileOutputStream fos = new FileOutputStream(target);
		final StreamResult streamResult = new StreamResult(fos);

		final Transformer transformer = Transformers.takeTransformer();
		if (contentNode.getNodeType() == Node.TEXT_NODE) {
			if (this.logger.isLoggable(Level.FINEST)) {
				this.logger.finest("process content as text");
			}
			transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION,
					"yes");
		} else {
			if (this.logger.isLoggable(Level.FINEST)) {
				this.logger.finest("process content as xml");
			}
			transformer
					.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "no");
			transformer.setOutputProperty(OutputKeys.METHOD, "xml");
		}

		try {
			transformer.transform(source, streamResult);
		} finally {
			fos.close();
			Transformers.releaseTransformer(transformer);
		}
	}
}
