/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.bc.filetransfer.util;

import static org.ow2.petals.bc.filetransfer.FileTransferConstants.BASE_MSG_ATTACH_VAR;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.BASE_MSG_CONTENT_VAR;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.ENUM_TRANSFER_MODE_ATTACHMENT;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.ENUM_TRANSFER_MODE_CONTENT;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.PARAM_BACKUP_DIRECTORY;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.PARAM_FOLDER;
import static org.ow2.petals.bc.filetransfer.FileTransferConstants.PARAM_POLLING_PERIOD;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.channels.FileChannel;
import java.nio.channels.FileLock;
import java.nio.channels.OverlappingFileLockException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.activation.DataHandler;
import javax.jbi.JBIException;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.transform.stream.StreamSource;

import org.ow2.petals.bc.filetransfer.FileTransferConstants;
import org.ow2.petals.bc.filetransfer.service.consume.ConsumeConfiguration;
import org.ow2.petals.component.framework.api.Message.MEPConstants;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.util.FileNamePatternUtil;
import org.ow2.petals.component.framework.util.MtomUtil;
import org.ow2.petals.component.framework.util.SourceUtil;
import org.ow2.petals.component.framework.util.XMLUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import com.ebmwebsourcing.easycommons.lang.StringHelper;
import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;
import com.ebmwebsourcing.easycommons.xml.XMLHelper;

/**
 * @author Roland NAUDIN - EBM WebSourcing
 * @author Mathieu CARROLLE - EBM WebSourcing
 */
public class FileTransferUtils {

    /**
     * Add the current time as suffix of the filename
     * 
     * @param filename
     * @return
     */
    public static final String addDateToFileName(String filename) {
        final String result;
        final long time = System.nanoTime();
        // Format the string to contains always 14 characters.
        final String sTime = String.format("%1$014d", time);
        int index = filename.lastIndexOf('.');
        if (index > 0) {
            String prefix = filename.substring(0, filename.lastIndexOf('.'));
            String suffix = filename.substring(filename.lastIndexOf('.'), filename.length());
            result = prefix + "-" + sTime + suffix;
        } else {
            result = filename + "-" + sTime;
        }

        return result;
    }

    /**
     * Send the file as an attachment
     * 
     * @param file
     * @throws MessagingException
     * @throws IOException
     */
    public static final void addFileAsAttachment(File file, String filename,
            NormalizedMessage message) throws MessagingException, IOException {
        DataHandler attachment = new DataHandler(file.toURI().toURL());
        message.addAttachment(filename, attachment);
    }

    /**
     * Build the configuration for consume service-unit. For non-mandatory
     * parameter, default value are set to parameter in this method.
     * 
     * @param extensions
     * @return ConsumeConfiguration instance
     * @throws MessagingException
     */
    public static ConsumeConfiguration buildConsumeConfiguration(ConfigurationExtensions extensions)
            throws MessagingException {
        // Specifed default value for consume configuration
        String baseMsg = null;
        boolean baseMsgOn = false;
        String transferMode = FileTransferConstants.ENUM_TRANSFER_MODE_ATTACHMENT;
        long pollingPeriod = FileTransferConstants.DEFAULT_POLLING_PERIOD;
        LockFileFilter regexFileNameFilter = null;
        File polledDirectory = null;
        File backupDirectory = null;
        int corePoolSize = FileTransferConstants.DEFAULT_CONSUME_POOL_SIZE;
        long corePoolTimeout = FileTransferConstants.DEFAULT_CONSUME_POOL_TIMEOUT;

        baseMsg = extensions.get(FileTransferConstants.PARAM_BASE_MSG);
        try {
            if (!StringHelper.isNullOrEmpty(baseMsg)) {
                transferMode = FileTransferUtils.checkTransferModeInBaseMsg(baseMsg);
                baseMsgOn = true;
            } else {
                transferMode = FileTransferUtils.checkTransferModeInSU(extensions
                        .get(FileTransferConstants.PARAM_TRANSFER_MODE));
            }
        } catch (MessagingException e) {
            // Default mode is used (File are processed as attachment with
            // default message)
        }

        final String s_pollingPeriod = extensions.get(PARAM_POLLING_PERIOD);
        if (!StringHelper.isNullOrEmpty(s_pollingPeriod)) {
            pollingPeriod = Long.parseLong(s_pollingPeriod);
        }
        // retrieve file name pattern
        final String filePattern = extensions.get(FileTransferConstants.PARAM_FILENAME);
        if (StringHelper.isNullOrEmpty(filePattern)) {
            regexFileNameFilter = new LockFileFilter();
        } else {
            regexFileNameFilter = new LockFileFilter(filePattern);
        }

        try {
            polledDirectory = FileTransferUtils.getFolderPath(extensions);
            backupDirectory = FileTransferUtils.getBackupDirectory(extensions);
        } catch (IOException e) {
            throw new MessagingException(e.getMessage());
        }
        try {
            corePoolSize = Integer.parseInt(extensions
                    .get(FileTransferConstants.PARAM_PROCESSOR_POOL_SIZE));
        } catch (NumberFormatException e) {
        }
        try {
            corePoolTimeout = Long.parseLong(extensions
                    .get(FileTransferConstants.PARAM_PROCESSOR_POOL_TIMEOUT));
        } catch (NumberFormatException e) {
        }
        return new ConsumeConfiguration(backupDirectory, baseMsg, baseMsgOn, 0,
                regexFileNameFilter, pollingPeriod, polledDirectory, transferMode, corePoolSize,
                corePoolTimeout);
    }

    /**
     * check if the file can be read
     * 
     * @param file
     * @return true if the file can be read
     */
    public static final boolean checkAccess(File file) {
        boolean result = false;
        FileChannel channel = null;
        FileLock lock = null;
        try {
            // try to access the file to test if it can be read
            channel = new RandomAccessFile(file, "rw").getChannel();
            // Use the file channel to create a lock on the file.
            // This method blocks until it can retrieve the lock.
            lock = channel.lock();
            lock.release();
            result = true;
        } catch (OverlappingFileLockException e) {
            // do nothing
        } catch (IOException e) {
            // do nothing
        } finally {
            if (channel != null) {
                try {
                    channel.close();
                } catch (IOException e1) {
                    // do nothing
                }

            }
        }
        return result;
    }

    /**
     * Extract the transfer mode variable in the base message. If no mode found
     * an exception is throw. Default variable value are : $content |
     * $attachment Override it to use custom behaviour.
     * 
     * @param baseMsg
     * @return
     * @throws MessagingException
     */
    public static String checkTransferModeInBaseMsg(final String baseMsg) throws MessagingException {
        String transferMode = null;
        if (baseMsg.contains(BASE_MSG_CONTENT_VAR))
            transferMode = ENUM_TRANSFER_MODE_CONTENT;
        else if (baseMsg.contains(BASE_MSG_ATTACH_VAR))
            transferMode = ENUM_TRANSFER_MODE_ATTACHMENT;
        else
            throw new MessagingException("Transfer mode not recognized in the base-msg : "
                    + baseMsg);
        return transferMode;
    }

    /**
     * Extract the transfer mode described in the jbi.xml of the service unit.
     * If no mode found an exception is throw. Override it to use custom
     * behaviour.
     * 
     * @param suTransferMode
     * @return
     * @throws MessagingException
     */
    public static String checkTransferModeInSU(final String suTransferMode)
            throws MessagingException {
        String transferMode = null;
        if (ENUM_TRANSFER_MODE_CONTENT.equalsIgnoreCase(suTransferMode)) {
            transferMode = ENUM_TRANSFER_MODE_CONTENT;
        } else if (ENUM_TRANSFER_MODE_ATTACHMENT.equalsIgnoreCase(suTransferMode)) {
            transferMode = ENUM_TRANSFER_MODE_ATTACHMENT;
        } else {
            throw new MessagingException("Transfer mode not recognized :" + suTransferMode);
        }
        return transferMode;
    }

    /**
     * Send the file as a Source
     * 
     * @param file
     * @param message
     * @throws IOException
     * @throws MessagingException
     */
    public static final void copyFileToContent(File file, NormalizedMessage message)
            throws IOException, MessagingException {
        StreamSource source = new StreamSource(new FileInputStream(file));
        message.setContent(source);
    }

    /***
     * Create message based on the xml structure define by the base-msg
     * parameter
     * 
     * @param processedFile
     *            The processed file.
     * @param fileName
     *            When processing file as attachment,filename is the content
     *            id(MTOM)
     * @param baseMsg
     *            The base msg.
     * @param transferMode
     *            Represent the transfer mode to use.(attachment or content)
     * @return The xml document build on the base message.
     * @throws IOException
     * @throws SAXException
     * @throws MessagingException
     */
    public static Document createMessageOnBaseMsg(final File processedFile, final String fileName,
            final String baseMsg, final String transferMode) throws IOException, MessagingException {
        Document processedFileAsDoc = null;
        Document finalMessage = null;
       
        Document baseMsgAsDoc = null;
        DocumentBuilder docBuilder = null;
        try {
            docBuilder = DocumentBuilders.takeDocumentBuilder();
            baseMsgAsDoc = docBuilder.parse(
                    new ByteArrayInputStream(baseMsg.getBytes()));
        } catch (SAXException e) {
            throw new MessagingException("The defined base message is invalid: [" + e.getMessage()
                    + "]");
        } finally {
            if(docBuilder != null) {
                DocumentBuilders.releaseDocumentBuilder(docBuilder);
            }
        }
        
        if (transferMode.equalsIgnoreCase(ENUM_TRANSFER_MODE_CONTENT)) {
            try {
                processedFileAsDoc = XMLUtil.loadDocument(new FileInputStream(processedFile));
            } catch (SAXException e) {
                throw new MessagingException(processedFile.getAbsolutePath()
                        + " is not a valid xml file.can't process it in content transfer mode");
            }
            finalMessage = replaceVariableByXmlFileContent(baseMsgAsDoc, processedFileAsDoc,
                    BASE_MSG_CONTENT_VAR);
        } else if (transferMode.equalsIgnoreCase(ENUM_TRANSFER_MODE_ATTACHMENT)) {
            finalMessage = replaceVariableByXopElement(fileName, baseMsgAsDoc, BASE_MSG_ATTACH_VAR);
        }
        
        return finalMessage;
    }

    /***
     * Create message based on default message structure.
     * 
     * @param processedFile
     *            The processed file.
     * @param fileName
     *            When processing file as attachment,filename is the content
     *            id(MTOM)
     * @param transferMode
     *            Represent the transfer mode to use.(attachment or content)
     * @return The xml document build on the default message.
     * @throws IOException
     * @throws MessagingException
     */
    public static Document createMessageOnDefaultMsg(final File processedFile,
            final String fileName, final String transferMode) throws IOException,
            MessagingException {
        Document inputXmlFile = null;
        Document finalMessage = null;
        if (transferMode.equals(ENUM_TRANSFER_MODE_CONTENT)) {
            try {
                inputXmlFile = XMLUtil.loadDocument(new FileInputStream(processedFile));
            } catch (SAXException e) {
                throw new MessagingException(processedFile.getAbsolutePath()
                        + " is not a valid xml file.can't process it in content transfer mode");
            }
            finalMessage = inputXmlFile;
        } else if (transferMode.equals(ENUM_TRANSFER_MODE_ATTACHMENT)) {
            finalMessage = injectAttachment(fileName);
        }
        return finalMessage;
    }

    /**
     * Evaluate a file path to resolve environment variables (place holders).
     */
    public static final String evaluateFilePath(String filePath) {
        String result = filePath;
        List<String> keys = extractPlaceHolderKeys(filePath);
        for (String key : keys) {
            if (!StringHelper.isNullOrEmpty(System.getenv(key))) {
                result = filePath.replace("${" + key + "}", System.getenv(key));
            }
        }
        // update file separator according to the OS dependent separator
        if (File.separator.equals("/")) {
            result = result.replace("\\", "/");
        } else {
            result = result.replace("/", "\\");
        }
        return result;
    }

    /**
     * extract in the exchange request the filepattern to use
     * 
     * @param exchange
     * @return
     * @throws MessagingException
     *             error while accessing the Exchange
     * @throws FileNotFoundException
     *             element not found in the XML request
     */
    public static String extractFilePattern(Exchange exchange) throws MessagingException,
            FileNotFoundException {
        Document doc = exchange.getInMessageContentAsDocument();
        Node rootNode = doc.getFirstChild();
        Node pattern = null;
        if (rootNode != null) {
            pattern = XMLHelper.findChild(rootNode, null, "filepattern", false);
        }
        if (pattern == null) {
            throw new FileNotFoundException("incorrect file filter found in filepattern node");
        }
        return pattern.getTextContent();
    }

    /***
     * Generate a MTOM response structure for a list of files.
     * 
     * @param names
     * @param operation
     * @return
     */
    public static final Document generateMTOMListResponse(List<String> names, QName operation) {
        Document doc = DocumentBuilders.newDocument();
        
        Element root = doc.createElementNS(operation.getNamespaceURI(),
                "tns:" + operation.getLocalPart() + "Response");
        root.appendChild(MtomUtil.generateMtomStructure(doc, names,
                new QName(operation.getNamespaceURI(), "attachments", "tns"),
                new QName(operation.getNamespaceURI(), "filename", "tns")));
        doc.appendChild(root);
        
        return doc;
    }

    /***
     * Generate a MTOM response structure for a single file.
     * 
     * @param names
     * @param operation
     * @return
     */
    public static Document generateMTOMResponse(String name, QName operation) {
        Document doc = DocumentBuilders.newDocument();
        
        Element root = doc.createElementNS(operation.getNamespaceURI(),
                "tns:" + operation.getLocalPart() + "Response");
        root.appendChild(MtomUtil.generateMtomStructure(doc, Arrays.asList(name), new QName(
                operation.getNamespaceURI(), "attachment", "tns"),
                new QName(operation.getNamespaceURI(), "filename", "tns")));
        doc.appendChild(root);
        
        return doc;
    }

    /**
     * Return the backup directory, {@code null} if it is not configured.
     * 
     * @return the backup directory
     */
    public static final File getBackupDirectory(ConfigurationExtensions extensions)
            throws IOException {
        File working = null;
        String directory = extensions.get(PARAM_BACKUP_DIRECTORY);
        if (!StringHelper.isNullOrEmpty(directory)) {
            directory = FileTransferUtils.evaluateFilePath(directory);
            working = new File(directory);
            if (!working.exists()) {
                throw new FileNotFoundException(PARAM_BACKUP_DIRECTORY + " does not exist :"
                        + directory);
            }
        }
        return working;
    }

    /**
     * Return the workspace directory
     * 
     * @return
     * @throws PEtALSCFException
     *             no working directory specified in the SU
     */
    public static final File getFolderPath(ConfigurationExtensions extensions) throws IOException {
        String directory = extensions.get(PARAM_FOLDER);
        if (StringHelper.isNullOrEmpty(directory)) {
            throw new FileNotFoundException("'" + PARAM_FOLDER + "' parameter is not set");
        }
        directory = FileTransferUtils.evaluateFilePath(directory);
        File working = new File(directory);
        if (!working.exists()) {
            throw new FileNotFoundException("'" + PARAM_FOLDER + "' parameter with value '"
                    + directory + "' does not exist");
        }
        return working;
    }

    /**
     * Build a message to indicate the mep available for this operation. TODO :
     * Move it in the cdk api.
     * 
     * @param operation
     * @param mep
     * @return
     */
    public static final String getValidMepMessageForOperation(final String operation,
            MEPConstants... mep) {
        StringBuilder builder = new StringBuilder();
        builder.append("Exchange pattern for the " + operation.toUpperCase()
                + " operation must be ");
        for (int i = 0; i < mep.length; i++) {
            if (i > 0)
                builder.append(" or ");
            builder.append(mep[i]);
        }
        return builder.toString();
    }

    /**
     * return the files contained in the READ DIRECTORY set in the extensions,
     * using the given filePattern
     * 
     * @param extensions
     * @return
     */
    public static final File[] listFiles(File readDir, String filePattern) {
        return readDir.listFiles(FileNamePatternUtil.getInstance().buildFileNameFilterFromWildChar(
                filePattern));
    }

    /**
     * move file to the backup directory specified in the SU<br>
     * if no backup directory if set, move to the Temp directory. <br>
     * The file need to be moved before being read
     * 
     * @param file
     * @return the moved file
     * @throws IOException
     */
    public static final File moveFile(File file, final ConfigurationExtensions extensions)
            throws IOException {
        // move file in a work or backup dir before processing
        File processedFile = null;
        final File backupDirectory = getBackupDirectory(extensions);
        if (backupDirectory == null) {
            File tmpDirectory = new File(System.getProperty("java.io.tmpdir"));
            processedFile = new File(tmpDirectory, "petals-bc-filetransfer-" + file.getName()
                    + System.nanoTime());
        } else {
            String target = "backup-" + addDateToFileName(file.getName());
            processedFile = new File(backupDirectory, target);
        }
        if (!file.renameTo(processedFile)) {
            throw new IOException("Permission denied.Can not move file " + file.getName() + " to "
                    + processedFile.getAbsolutePath());
        }
        return processedFile;
    }

    /**
     * TODO : create the fault from its WSDL description.
     * 
     * @param exchange
     * @param ioMessage
     * @throws JBIException
     */
    public static final void setIOFaultOnExchange(Exchange exchange, String ioMessage)
            throws MessagingException {
        StringBuffer faultMessage = new StringBuffer();
        faultMessage.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
        faultMessage.append("<tns:ioFault xmlns:tns=\"");
        faultMessage.append(FileTransferConstants.FILETRANSFER_SERVICE_NS);
        faultMessage.append("\"><tns:message>");
        faultMessage.append(ioMessage);
        faultMessage.append("</tns:message></tns:ioFault>");
        Fault fault = exchange.createFault();
        try {
            // Create a source in UTF-8 format
            fault.setContent(SourceUtil.createSource(faultMessage.toString(),
                    "UTF-8"));
            exchange.setFault(fault);
        } catch (PEtALSCDKException e) {
            throw new MessagingException(e.getMessage());
        }
    }

    public static final void setMissingElementFaultOnExchange(Exchange exchange,
            String missingElement) throws MessagingException {
        final StringBuffer faultMessage = new StringBuffer();
        faultMessage.append("<tns:missingElementFault xmlns:tns='");
        faultMessage.append(FileTransferConstants.FILETRANSFER_SERVICE_NS);
        faultMessage.append("'><tns:element>");
        faultMessage.append(missingElement);
        faultMessage.append("</tns:element></tns:missingElementFault>");
        Fault fault = exchange.createFault();
        try {
            fault.setContent(SourceUtil.createSource(faultMessage.toString()));
            exchange.setFault(fault);
        } catch (PEtALSCDKException e) {
            throw new MessagingException(e.getMessage());
        }
    }

    /**
     * Extract the place holders.
     * 
     * @param entryValue
     * @return List of place holders
     */
    private static final List<String> extractPlaceHolderKeys(String entryValue) {
        List<String> result = new ArrayList<String>();
        String[] keys = entryValue.split("[${]");
        for (String string : keys) {
            int index = string.indexOf("}");
            if (index > 0) {
                result.add(string.substring(0, index));
            }
        }
        return result;
    }

    /***
     * Inject in the default message structure, the xop:Include element with
     * contenId equals to filename.
     * <p>
     * Override it for a custom default message.
     * </p>
     * 
     * @param fileName
     *            To specify the content id used by xop:Include element.
     */
    private static Document injectAttachment(final String fileName) {
        Document doc = DocumentBuilders.newDocument();
        
        doc.appendChild(MtomUtil.generateMtomStructure(doc, Arrays
                .asList(fileName), new QName(
                "http://petals.ow2.org/components/filetransfer/version-3", "attachment")));
        
        return doc;
    }

    /***
     * Inject in the base message, the processed xml file content. This element
     * replace the variable found.
     * 
     * @param baseMsgDoc
     * @param processedFileDoc
     * @param variable
     * @return
     * @throws MessagingException
     */
    private static final Document replaceVariableByXmlFileContent(Document baseMsgDoc,
            Document processedFileDoc, String variable) throws MessagingException {
        final Node node = XMLUtil.findVariable(baseMsgDoc.getChildNodes(), variable);
        if (node == null) {
            throw new MessagingException(variable + " variable not found in the base message.");
        }
        final Node parentNode = node.getParentNode();
        final Node importedNode = baseMsgDoc
                .importNode(processedFileDoc.getDocumentElement(), true);
        parentNode.replaceChild(importedNode, node);
        return baseMsgDoc;
    }

    /***
     * Inject in the base message, the xop:Include element with contenId equals
     * to filename. This element replace the variable found.
     * 
     * @param filename
     * @param baseMsgDoc
     * @param variable
     * @return
     * @throws MessagingException
     */
    private static final Document replaceVariableByXopElement(final String filename,
            final Document baseMsgDoc, final String variable) throws MessagingException {
        if (baseMsgDoc != null) {
            final Node node = XMLUtil.findVariable(baseMsgDoc.getChildNodes(), variable);
            if (node == null) {
                throw new MessagingException(variable + " variable not found in the base message.");
            }
            final Node parentNode = node.getParentNode();
            parentNode.replaceChild(MtomUtil.generateMtomXopElement(baseMsgDoc, filename), node);
        }
        return baseMsgDoc;
    }
}
