/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.bc.filetransfer.util;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;

/**
 * Utility class to handle the transfer report.
 * @author Mathieu CARROLLE - EBM WebSourcing
 * @author Roland Naudin - EBM WebSourcing
 */
public class TransferReport {

    private Map<File, Exception> transferedFiles;

    public static final String NAMESPACE = "http://petals.ow2.org/component/filetransfer/report";

    public static final String ROOT_ELEMENT = "transferReport";

    public static final String FILE_ELEMENT = "file";

    public static final String FILENAME_ATTRIBUTE = "name";

    public static final String TRANSFER_ATTRIBUTE = "transfer";

    public static final String FILENAME_OK = "OK";

    public static final String FILENAME_KO = "KO";

    public static final String EXCEPTION_ELEMENT = "Exception";

    private boolean failedTransfer = false;

    /**
     * Default constructor
     * 
     * @throws ParserConfigurationException
     */
    public TransferReport() {
        transferedFiles = new HashMap<File, Exception>();
    }

    /**
     * Add a transfered file to the report.
     * 
     * @param file
     *            The file object
     * @param exception
     *            The exception if the transfer has failed, {@code null}
     *            otherwise
     */
    public void addTransferedFile(File file, Exception exception) {
        if (file == null) {
            throw new IllegalArgumentException("The file parameter must be non null");
        }

        if (exception != null) {
            failedTransfer = true;
        }
        transferedFiles.put(file, exception);
    }

    public String generateRawReport() {
        StringBuilder report = new StringBuilder();

        for (File transferedFile : transferedFiles.keySet()) {
            report.append("File: ").append(transferedFile.getAbsolutePath())
                    .append(" - transfer status: ");
            if (transferedFiles.get(transferedFile) == null) {
                report.append(FILENAME_OK);
            } else {
                report.append(FILENAME_KO).append(
                        ". Cause: " + transferedFiles.get(transferedFile).getMessage());
            }
            report.append(".\n");
        }

        return report.toString();
    }

    /**
     * Generate the report of the transfers done.
     * 
     * @return the document describing the report
     */
    public Document generateXmlReport() {
        Document reportDocument = DocumentBuilders.newDocument();
        
        Element root = reportDocument.createElementNS(NAMESPACE, ROOT_ELEMENT);
        reportDocument.appendChild(root);

        for (File transferedFile : transferedFiles.keySet()) {
            Element fileTransfered = reportDocument.createElement(FILE_ELEMENT);
            root.appendChild(fileTransfered);
            fileTransfered.setAttribute(FILENAME_ATTRIBUTE, transferedFile.getAbsolutePath());
            if (transferedFiles.get(transferedFile) == null) {
                fileTransfered.setAttribute(TRANSFER_ATTRIBUTE, FILENAME_OK);
            } else {
                fileTransfered.setAttribute(TRANSFER_ATTRIBUTE, FILENAME_KO);
                Element exception = reportDocument.createElement(EXCEPTION_ELEMENT);
                exception.setTextContent(transferedFiles.get(transferedFile).getMessage());
                fileTransfered.appendChild(exception);
            }
        }

        return reportDocument;
    }

    public boolean containsFailedTransfer() {
        return failedTransfer;
    }
}
